
UNIT YAH_CLAS;

(* GESTION DES CLASSEMENTS SUR DISQUE POUR LE PROGRAMME YAHTZEE 2.20 *)

INTERFACE

{$I-}

Uses DOS,TXT,YAH_HARD,YAH_DECL,YAH_DLGS;

Procedure SAUVETAGE_CLASSEMENTS;
   (* CLASSEMENT ET SAUVETAGE EVENTUEL DES RESULTATS DANS LES FICHIERS DE CLASSEMENT *)


IMPLEMENTATION

Function LECTEUR_COURANT : byte;
(* RENVOIE LE CODE DU DRIVE COURANT *)
(* CONSERVEE VERSION 2.20 *)
Var Registres:registers;
Begin
  Registres.ah:=$19;
  intr($21,Registres);
  LECTEUR_COURANT:=Registres.al;
End;


Function PRESENCE_FICHIER (NomFichier:string) : byte;
(* TESTE LA PRESENCE D'UN FICHIER DANS LE REPERTOIRE COURANT.
   VALEURS RETOURNEES : 0 = FICHIER PRESENT, 1 = FICHIER ABSENT, 2 = AVORTER *)
(* MODIFIEE VERSION 2.20 *)
Var s:searchrec;
    Ok:boolean;
    Erreur:integer;
Begin
  PRESENCE_FICHIER:=0;
  Ok:=false;
  repeat
    findfirst(NomFichier,archive,s);
    Erreur:=doserror;
    case Erreur of
      0:Ok:=true;
      3,152,156,158,161,162:if not MESSAGE_ERREUR(0) then begin PRESENCE_FICHIER:=2; Ok:=true; end;
    else
      PRESENCE_FICHIER:=1; Ok:=true;
    end;
  until Ok;
End;


Function CREATION_FICHIER : boolean;
(* CREATION D'UN FICHIER DE CLASSEMENT (VARIABLE GLOBALE Fichier).
   LA FONCTION RETOURNE true SI OK. *)
(* MODIFIEE VERSION 2.20 *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    rewrite(Fichier); Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(1);
  until (Ok) or (Avorter);
  CREATION_FICHIER:=Ok;
End;


Function OUVERTURE_FICHIER : boolean;
(* OUVERTURE D'UN FICHIER DE CLASSEMENT (VARIABLE GLOBALE Fichier).
   LA FONCTION RETOURNE true SI OK. *)
(* MODIFIEE VERSION 2.20 *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false; Erreur:=ioresult;
  repeat
    reset(Fichier);
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(2);
  until (Ok) or (Avorter);
  OUVERTURE_FICHIER:=Ok;
End;


Function LECTURE_FICHIER : boolean;
(* LECTURE D'UN FICHIER DE CLASSEMENT (VARIABLE GLOBALE Fichier).
   LA FONCTION RETOURNE true SI OK. *)
(* MODIFIEE VERSION 2.20 *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    read(Fichier,Buffer); Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:=not MESSAGE_ERREUR(3);
  until (Ok) or (Avorter);
  LECTURE_FICHIER:=Ok;
End;


Function ECRITURE_FICHIER : boolean;
(* ECRITURE DANS UN FICHIER DE CLASSEMENT (VARIABLE GLOBALE Fichier).
   LA FONCTION RETOURNE true SI OK. *)
(* MODIFIEE VERSION 2.20 *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    write(Fichier,Buffer); Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:= not MESSAGE_ERREUR(4);
  until (Ok) or (Avorter);
  ECRITURE_FICHIER:=Ok;
End;


Function FERMETURE_FICHIER : boolean;
(* FERMETURE D'UN FICHIER DE CLASSEMENT (VARIABLE GLOBALE Fichier).
   LA FONCTION RETOURNE true SI OK. *)
(* MODIFIEE VERSION 2.20 *)
Var Ok,Avorter:boolean;
    Erreur:integer;
Begin
  Ok:=false; Avorter:=false;
  repeat
    close(Fichier); Erreur:=ioresult;
    if Erreur = 0 then Ok:=true else Avorter:= not MESSAGE_ERREUR(5);
  until (Ok) or (Avorter);
  FERMETURE_FICHIER:=Ok;
End;


Procedure SAUVETAGE_CLASSEMENTS;
(* CLASSEMENT ET SAUVETAGE EVENTUEL DES RESULTATS DANS LES FICHIERS DE CLASSEMENT *)
(* MODIFIEE VERSION 2.20 *)
Var Drive,Resultat,Joueur,n,m:byte;
    Jour,Mois,Annee,JourSemaine:word;
    Chaine:string[4];
    Ok,Fin:boolean;
    Entree:char;
Begin
  (* TEST DE LA PRESENCE DU FICHIER *)
  Drive:=LECTEUR_COURANT; if Drive <= 1 then INSERER_DISQUETTE;
  (* TRAITEMENT DU FICHIER YAHTZEE1.RES (SANS PRIMES YAHTZEE) *)
  NomFichier:='YAHTZEE1.RES'; assign(Fichier,NomFichier);
  Resultat:=PRESENCE_FICHIER(NomFichier);
  Fin:=false;
  case Resultat of
    0:begin   (* LE FICHIER EXISTE : CHARGEMENT DU CLASSEMENT *)
        for Joueur:=1 to 20 do
          with Classement1[Joueur] do
            begin
              NomJoueur:=''; Date:='00-00-0000'; Total:=0; TotalPrimes:=0;
            end;
        if OUVERTURE_FICHIER
           then
             begin
               Joueur:=1; Ok:=true;
               while (not eof(Fichier)) and (Ok) do
                 begin
                   Ok:=LECTURE_FICHIER; if Ok then Classement1[Joueur]:=Buffer; inc(Joueur);
                 end;
               if not Ok
                  then
                    begin
                      Void:=FERMETURE_FICHIER; Fin:=true;
                    end;
             end
           else
             Fin:=true;
      end;
    1:begin   (* LE FICHIER N'EXISTE PAS : LE TABLEAU EST CREE VIDE *)
        for Joueur:=1 to 20 do
          with Classement1[Joueur] do
            begin
              NomJoueur:=''; Date:='00-00-0000'; Total:=0; TotalPrimes:=0;
            end;
      end;
    2:Fin:=true;
  end;
  if not Fin
     then
       begin
         (* TRAITEMENT DU FICHIER YAHTZEE2.RES (AVEC PRIMES YAHTZEE) *)
         NomFichier:='YAHTZEE2.RES'; assign(Fichier,NomFichier);
         Resultat:=PRESENCE_FICHIER(NomFichier);
         case Resultat of
           0:begin   (* LE FICHIER EXISTE *)
               for Joueur:=1 to 20 do
                 with Classement2[Joueur] do
                   begin
                     NomJoueur:=''; Date:='00-00-0000'; Total:=0; TotalPrimes:=0;
                   end;
               if OUVERTURE_FICHIER
                  then
                    begin
                      Joueur:=1; Ok:=true;
                      while (not eof(Fichier)) and (Ok) do
                        begin
                          Ok:=LECTURE_FICHIER; if Ok then Classement2[Joueur]:=Buffer; inc(Joueur);
                        end;
                      if not Ok
                         then
                           begin
                             Void:=FERMETURE_FICHIER; Fin:=true;
                           end;
                    end
                  else
                    Fin:=true;
             end;
           1:begin   (* LE FICHIER N'EXISTE PAS : LE TABLEAU EST CREE VIDE *)
               for Joueur:=1 to 20 do
                 with Classement2[Joueur] do
                   begin
                     NomJoueur:=''; Date:='00-00-0000'; Total:=0; TotalPrimes:=0;
                   end;
             end;
           2:Fin:=true;
         end;
       end;
  if not Fin
     then
       begin
         (* AJOUT EVENTUEL DES RESULTATS DU JEU DANS LE CLASSEMENT 1 (SANS PRIMES) *)
         for Joueur:=1 to NbJoueurs do
           begin
             n:=1;
             while n <= 20 do
               begin
                 if FeuillePoints[Joueur,fp_Total,1] > Classement1[n].Total
                    then
                      begin   (* INSERTION DANS LE CLASSEMENT *)
                        for m:=20 downto (n + 1) do Classement1[m]:=Classement1[m - 1];
                        with Classement1[n] do
                          begin
                            NomJoueur:=NomsJoueurs[Joueur];
                            getdate(Annee,Mois,Jour,JourSemaine);
                            str(Jour,Chaine); if length(Chaine) = 1 then insert('0',Chaine,1); Date:=Chaine;
                            str(Mois,Chaine); if length(Chaine) = 1 then insert('0',Chaine,1); Date:=Date + '-' + Chaine;
                            str(Annee,Chaine); Date:=Date + '-' + Chaine;
                            Total:=FeuillePoints[Joueur,fp_Total,1];
                            TotalPrimes:=FeuillePoints[Joueur,fp_Total,1] + TableauPrimes[Joueur];
                          end;
                        n:=21;   (* FIN DE BOUCLE *)
                      end
                    else
                      inc(n);
               end;
           end;
         (* AJOUT EVENTUEL DES RESULTATS DU JEU DANS LE CLASSEMENT 2 (AVEC PRIMES) *)
         for Joueur:=1 to NbJoueurs do
           begin
             n:=1;
             while n <= 20 do
               begin
                 if (FeuillePoints[Joueur,fp_Total,1] + TableauPrimes[Joueur]) > Classement2[n].TotalPrimes
                    then
                      begin   (* INSERTION DANS LE CLASSEMENT *)
                        for m:=20 downto (n + 1) do Classement2[m]:=Classement2[m - 1];
                        with Classement2[n] do
                          begin
                            NomJoueur:=NomsJoueurs[Joueur];
                            getdate(Annee,Mois,Jour,JourSemaine);
                            str(Jour,Chaine); if length(Chaine) = 1 then insert('0',Chaine,1); Date:=Chaine;
                            str(Mois,Chaine); if length(Chaine) = 1 then insert('0',Chaine,1); Date:=Date + '-' + Chaine;
                            str(Annee,Chaine); Date:=Date + '-' + Chaine;
                            Total:=FeuillePoints[Joueur,fp_Total,1];
                            TotalPrimes:=FeuillePoints[Joueur,fp_Total,1] + TableauPrimes[Joueur];
                          end;
                        n:=21;   (* FIN DE BOUCLE *)
                      end
                    else
                      inc(n);
               end;
           end;
         (* AFFICHAGE DES CLASSEMENTS A L'ECRAN *)
         AFFICHAGE_CLASSEMENTS;
         (* REECRITURE DES FICHIERS *)
         NomFichier:='YAHTZEE1.RES'; assign(Fichier,NomFichier);
         if CREATION_FICHIER
            then
              begin
                n:=1; Fin:=false;
                while (n <= 20) and (not Fin) do
                  begin
                    if Classement1[n].Total > 0
                       then
                         begin
                           Buffer:=Classement1[n]; Fin:=not ECRITURE_FICHIER;
                         end
                       else
                         Fin:=true;
                    inc(n);
                  end;
                Void:=FERMETURE_FICHIER;
              end;
         NomFichier:='YAHTZEE2.RES'; assign(Fichier,NomFichier);
         if CREATION_FICHIER
            then
              begin
                n:=1; Fin:=false;
                while (n <= 20) and (not Fin) do
                  begin
                    if Classement2[n].Total > 0
                       then
                         begin
                           Buffer:=Classement2[n]; Fin:=not ECRITURE_FICHIER;
                         end
                       else
                         Fin:=true;
                    inc(n);
                  end;
                Void:=FERMETURE_FICHIER;
              end;
       end;
End;

END.
